﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;
using System.Windows.Forms;
using System.IO;                //need this for open file dialog box
//using System.Windows.Controls;  //need this for open file dialog box ... go to add references, and then add PresentationFramework. Otherwise this will not work! 
using HidLibrary;
using Scale;
using System.Timers;



namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        const string fileStartingDirectory = "c:\\";
        const int arraySize = 31;
        decimal[] colourArray = new decimal[arraySize];
        string[] colourNameArray = new string[arraySize] 
        { 
            "Mixed Yellow",
            "Mixed Red",
            "Mixed Blue",
            "Mixed Gray",
            "Mixed Brown",
            "Mixed Purple and Violet",
            "Mixed Green",
            "Mixed Orange",
            "Yellow",
            "Rubine",
            "Cyan",
            "Black",
            "Film Black",
            "Film Reflex Blue",
            "Lightfast Yellow",
            "Lightfast Rubine",
            "DT Yellow",
            "DT Green",
            "877 Silver",
            "Trans White",
            "Opaque White",
            "021 Orange",
            "032 Red",
            "Warm Red",
            "Reflex Blue",
            "Green",
            "Purple",
            "Violet",
            "Rhodamine",
            "871 Gold",
            "872 Gold",
        };
 
        public decimal? startingLb;
        Boolean tareCurrentPounds = false;
        Boolean scaleCheck = false;

        public Form1()
        {
            InitializeComponent();
        }

        private void goButton_Click(object sender, EventArgs e)
        {
            USBScale s2 = new USBScale();
            decimal? weightInLb, weightInG, weightInOz;
            bool? isStable;

            //Make sure scale is connected, if so, set up the timer and get things rolling! 
            if (scaleCheck == false)
            {
                MessageBox.Show("No scale connected.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
            }
            else
            {
                timer1.Enabled = true;
                resetButton.Enabled = true;
                goButton.Enabled = false;
                Thread.Sleep(100);
                s2.Connect();
                s2.GetWeight(out weightInLb, out weightInG, out weightInOz, out isStable);
                startingLb = weightInLb;
                s2.Disconnect();        
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {

            decimal? weightInLb, weightInG, weightInOz;
            bool? isStable;
            USBScale s = new USBScale();
            s.Connect();

            if (s.IsConnected)
            {
                s.GetWeight(out weightInLb, out weightInG, out weightInOz, out isStable);
                //s.DebugScaleData(); // This writes the contents of the array retrieved from the scale to the console. LEGACY CODE! Code is still intact in Scale.cs
                poundsLabel.Text = System.Convert.ToString(weightInLb);
                gramsLabel.Text = System.Convert.ToString(weightInG);
                kgLabel.Text = System.Convert.ToString(weightInG / 1000);
                ozLabel.Text = System.Convert.ToString(weightInOz);
                if (tareCurrentPounds == true)
                {
                    startingLb = weightInLb;
                    tareCurrentPounds = false;
                }
                currentPoundsLabel.Text = System.Convert.ToString(weightInLb - startingLb);
                timer2.Enabled = false; // Don't need to keep checking whether or not scale is attached since this method will instead!
                s.Disconnect();

            }
            else
            {
                resetButton_Click(sender, e);
                MessageBox.Show("The scale was disconnected!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                timer2.Enabled = true; // Start periodically checking to see whether or not the scale has been reconnected
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            UpdateLabels();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            tareCurrentPounds = true;
        }

        private void resetButton_Click(object sender, EventArgs e)
        {
            //Disable the resetButton, enable the goButton, viewTableButton, scaleReaderButton
            resetButton.Enabled = false;
            goButton.Enabled = true;

            //Disable the timer to stop the scale values from updating
            timer1.Enabled = false;

            //Reset all the labels displaying mass
            gramsLabel.Text = "0.00";
            kgLabel.Text = "0.00";
            ozLabel.Text = "0.00";
            poundsLabel.Text = "0.00";
            currentPoundsLabel.Text = "0.00";
        }

        private void timer2_Tick(object sender, EventArgs e)
        {
            USBScale s3 = new USBScale();
            s3.Connect();

            if (s3.IsConnected)
            {
                scaleCheck = true;
                s3.Disconnect();
            }
            else
            {
                scaleCheck = false;
            }

        }

        private void btnResetTotals_Click(object sender, EventArgs e)
        {
            ResetTotals();
        }

        private void ResetTotals()
        {
            //Set each value in the array to 0
            for (int x = 0; x < arraySize; x++)
                colourArray[x] = 0;

            //Update the labels
            UpdateLabels();
        }

        private void UpdateLabels()
        {
            //Update the mixed colour totals
            lblYellowMix.Text = Convert.ToString(colourArray[0]);
            lblRedMix.Text = Convert.ToString(colourArray[1]);
            lblBlueMix.Text = Convert.ToString(colourArray[2]);
            lblGrayMix.Text = Convert.ToString(colourArray[3]);
            lblBrownMix.Text = Convert.ToString(colourArray[4]);
            lblPurpMix.Text = Convert.ToString(colourArray[5]);
            lblGreenMix.Text = Convert.ToString(colourArray[6]);
            lblOrangeMix.Text = Convert.ToString(colourArray[7]);

            //Update the base colour totals
            lblYellow.Text = Convert.ToString(colourArray[8]);
            lblRubine.Text = Convert.ToString(colourArray[9]);
            lblCyan.Text = Convert.ToString(colourArray[10]);
            lblBlack.Text = Convert.ToString(colourArray[11]);
            lblFBlack.Text = Convert.ToString(colourArray[12]);
            lblFReflex.Text = Convert.ToString(colourArray[13]);
            lblLFYellow.Text = Convert.ToString(colourArray[14]);
            lblLFRubine.Text = Convert.ToString(colourArray[15]);
            lblDTYellow.Text = Convert.ToString(colourArray[16]);
            lblDTGreen.Text = Convert.ToString(colourArray[17]);
            lbl877Silver.Text = Convert.ToString(colourArray[18]);

            lblTransWhite.Text = Convert.ToString(colourArray[19]);
            lblOpWhite.Text = Convert.ToString(colourArray[20]);
            lbl021Orange.Text = Convert.ToString(colourArray[21]);
            lbl032Red.Text = Convert.ToString(colourArray[22]);
            lblWarmRed.Text = Convert.ToString(colourArray[23]);
            lblReflexBlue.Text = Convert.ToString(colourArray[24]);
            lblGreen.Text = Convert.ToString(colourArray[25]);
            lblPurple.Text = Convert.ToString(colourArray[26]);
            lblViolet.Text = Convert.ToString(colourArray[27]);
            lblRhodamine.Text = Convert.ToString(colourArray[28]);
            lbl871Gold.Text = Convert.ToString(colourArray[29]);
            lbl872Gold.Text = Convert.ToString(colourArray[30]);
        }



        private void UpdateColour(int colour)
        {
            //Update the total colour mass
            if (radioPlus.Checked == true)
                colourArray[colour] = colourArray[colour] + Convert.ToDecimal(currentPoundsLabel.Text);
            else if (radioMinus.Checked == true)
                colourArray[colour] = colourArray[colour] - Convert.ToDecimal(currentPoundsLabel.Text);

            //Update the labels
            UpdateLabels();
        }

        private void btnYellowMix_Click(object sender, EventArgs e)
        {
            UpdateColour(0);
        }

        private void btnRedMix_Click(object sender, EventArgs e)
        {
            UpdateColour(1);
        }

        private void buttonBlueMix_Click(object sender, EventArgs e)
        {
            UpdateColour(2);
        }

        private void btnGrayMix_Click(object sender, EventArgs e)
        {
            UpdateColour(3);
        }

        private void btnBrownMix_Click(object sender, EventArgs e)
        {
            UpdateColour(4);
        }

        private void btnPurpMix_Click(object sender, EventArgs e)
        {
            UpdateColour(5);
        }

        private void btnGreenMix_Click(object sender, EventArgs e)
        {
            UpdateColour(6);
        }

        private void btnOrangeMix_Click(object sender, EventArgs e)
        {
            UpdateColour(7);
        }

        private void btnYellow_Click(object sender, EventArgs e)
        {
            UpdateColour(8);
        }

        private void btnRubine_Click(object sender, EventArgs e)
        {
            UpdateColour(9);
        }

        private void btnCyan_Click(object sender, EventArgs e)
        {
            UpdateColour(10);
        }

        private void btnBlack_Click(object sender, EventArgs e)
        {
            UpdateColour(11);
        }

        private void btnFBlack_Click(object sender, EventArgs e)
        {
            UpdateColour(12);
        }

        private void btnFilmRBlue_Click(object sender, EventArgs e)
        {
            UpdateColour(13);
        }

        private void btnTransWhite_Click(object sender, EventArgs e)
        {
            UpdateColour(19);
        }

        private void btnOpWhite_Click(object sender, EventArgs e)
        {
            UpdateColour(20);
        }

        private void btnOrange_Click(object sender, EventArgs e)
        {
            UpdateColour(21);
        }

        private void btnRed_Click(object sender, EventArgs e)
        {
            UpdateColour(22);
        }

        private void btnWarmRed_Click(object sender, EventArgs e)
        {
            UpdateColour(23);
        }

        private void btnReflexBlue_Click(object sender, EventArgs e)
        {
            UpdateColour(24);
        }

        private void btnGreen_Click(object sender, EventArgs e)
        {
            UpdateColour(25);
        }

        private void btnPurple_Click(object sender, EventArgs e)
        {
            UpdateColour(26);
        }

        private void btnViolet_Click(object sender, EventArgs e)
        {
            UpdateColour(27);
        }

        private void btnRhodamine_Click(object sender, EventArgs e)
        {
            UpdateColour(28);
        }

        private void btnLFYellow_Click(object sender, EventArgs e)
        {
            UpdateColour(14);
        }

        private void btnLFRubine_Click(object sender, EventArgs e)
        {
            UpdateColour(15);
        }

        private void btnDTYellow_Click(object sender, EventArgs e)
        {
            UpdateColour(16);
        }

        private void btnDTGreen_Click(object sender, EventArgs e)
        {
            UpdateColour(17);
        }

        private void btnOneGold_Click(object sender, EventArgs e)
        {
            UpdateColour(29);
        }

        private void btnTwoGold_Click(object sender, EventArgs e)
        {
            UpdateColour(30);
        }

        private void btnSilver_Click(object sender, EventArgs e)
        {
            UpdateColour(18);
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            Stream fileStream = null;

            List<string> lines = new List<string>();

            //Create an instance of the dialog box
            OpenFileDialog openFileDialog1 = new OpenFileDialog();

            //Set filter options
            openFileDialog1.Filter = "Text Files(.txt)|*.txt";
            openFileDialog1.FilterIndex = 2;

            openFileDialog1.InitialDirectory = fileStartingDirectory;
            openFileDialog1.Multiselect = false;

            //Process input
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    string fileName = openFileDialog1.FileName;

                    string[] colourArrayTemp = new string[arraySize*2];

                    if ((fileStream = openFileDialog1.OpenFile()) != null)
                    {
                        using (StreamReader r = new StreamReader(fileName))
                        {
                            string line;
                            while((line = r.ReadLine()) != null)
                            {
                                lines.Add(line);
                            }
                        }

                        int x = 0;

                        foreach(string s in lines)
                        {
                            colourArrayTemp[x] = s;
                            x++;
                        }

                        //I get an error when I try to fill colourArray in the previous step, so it fills colourArray from another, temporary array that has twice the array size
                        int y = 1;
                        for (int z = 0; z < arraySize; z++)
                        {
                            colourArray[z] = Convert.ToDecimal(colourArrayTemp[y]);
                            y++;
                            y++;
                        }

                        UpdateLabels();

                        MessageBox.Show("File loaded!");
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error reading file: " + ex.Message);
                }
            }
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            List<string> lines = new List<string>();

            //Create an instance of the dialog box
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            //Set filter options
            saveFileDialog1.Filter = "Text Files(.txt)|*.txt";
            saveFileDialog1.FilterIndex = 2;

            saveFileDialog1.InitialDirectory = fileStartingDirectory;

            //Process input
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    string fileName = saveFileDialog1.FileName;

                    //Make another array that is twice the size of arraySize and fill it with the alternating values of colourNameArray and colourArray. 
                    string[] colourArrayTemp = new string[arraySize * 2];

                    int y = 0;

                    for(int x = 0; x < arraySize; x++)
                    {
                        colourArrayTemp[y] = colourNameArray[x];
                        colourArrayTemp[y+1] = Convert.ToString(colourArray[x]);
                        y++;
                        y++;
                    }

                    //Write the text file
                    System.IO.File.WriteAllLines(fileName, colourArrayTemp);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error saving file: " + ex.Message);
                }

                MessageBox.Show("File saved!");
            }

        }
    }
}
